# coding: utf-8
import os
from contextlib import contextmanager
from datetime import (
    datetime)

from lxml import (
    etree,
    objectify)

from petl.compat import (
    OrderedDict)
from petl.transform.maps import (
    fieldmap)

from .utils import xml_escape


EMPTY = (None, '')
UNBIND = ('-1', -1)
FALSE = (0, '0', False)

def SKIP(_):
    return True


def row(value):
    return value


def just(value):
    return lambda _: value


def maybe(mapping, value=None):
    return lambda v: mapping.get(v, value)


def first(mapping, value=None):
    return lambda arr: mapping.get(min(arr), value) if arr else None


def ns(namespace, element):
    return "{{{0}}}{1}".format(
            namespace, element)


def d(value):
    return value and value.date().isoformat() or None


def dt(value):
    return value and datetime.combine(value, datetime.min.time()).isoformat() or None


class Hidden(object):

    def __len__(self):
        return 0


def simple_serializer(obj):
    return objectify.DataElement(xml_escape(obj)).text or ''


class VirtualQuerySetWrapper(object):
    """Обертка для RawQuerySet.
    Имитирует поведение методов values_list и iterator.
    Позволяет использовать RawQuerySet в функции fromorm."""

    def __init__(self, rows):
        self.rows = rows

    def values_list(self, *fields):
        return self

    def iterator(self):
        for row in self.rows:
            yield row


class RawQuerySetWrapper(object):
    """Обертка для RawQuerySet.
    Имитирует поведение методов values_list и iterator.
    Позволяет использовать RawQuerySet в функции fromorm."""

    def __init__(self, model, options):
        self.model = model
        self.options = options
        self.fields = []

    def values_list(self, *fields):
        self.fields = fields

        return self

    def iterator(self):
        for row in self.model.objects.raw(*self.options):
            yield tuple([getattr(row, column) for column in self.fields])


def fromorm(qs, *fields):
    """Обертка для Django ORM QuerySet"""

    yield fields

    for row in qs.values_list(*fields).iterator():
        yield row


@contextmanager
def xfelement(xf, obj):
    if obj:
        if isinstance(obj, tuple):
            obj, opts = obj
        else:
            obj, opts = obj, {}

        with xf.element(obj, **opts):
            yield
    else:
        yield


def process_mapping(mapping, **kwargs):
    """Процессинг маппинга"""

    if callable(mapping):
        mapping = mapping(**kwargs)

    for (model, record, variants) in mapping:
        skip_header = False

        for (qs, rules) in variants:
            _rules = OrderedDict()
            options = []
            for rule_item in rules:
                k = rule_item[0]
                v = rule_item[1]

                if not k:
                    k = Hidden()

                _rules[k or Hidden()] = v

                options.append(rule_item[2:])

            fields = []

            for v in _rules.itervalues():
                if isinstance(v, str):
                    fields.append(v)
                elif isinstance(v, tuple) and len(v) == 2:
                    fields.append(v[0])

            header = True

            for row in fieldmap(fromorm(qs, *fields), _rules, True):
                if not (header and skip_header):
                    yield (model, record, row, options)

                header = False

            skip_header = True

        yield None


def fields_writer(xf, serializer, header, row, options, **kwargs):
    for (field, data, options) in zip(header, row, options):
        try:
            optionality_config = options[0]
        except IndexError:
            pass
        else:
            if isinstance(optionality_config, tuple):
                if data in optionality_config:
                    continue

            if callable(optionality_config):
                if optionality_config(data):
                    continue

        try:
            nested = options[1]
        except IndexError:
            nested = None

        if nested:
            with xfelement(xf, field):
                model_writer(xf, serializer, process_mapping(
                    nested(data, **kwargs), **kwargs), **kwargs)
        else:
            with xfelement(xf, field):
                xf.write(serializer(data))


def record_writer(xf, serializer, header, data, options, **kwargs):
    (model, record, row, options) = data

    with xfelement(xf, record):
        fields_writer(xf, serializer, header, row, options, **kwargs)


def model_writer(xf, serializer, iterator, **kwargs):
    for (model, record, header, options) in iterator:
        with xfelement(xf, model):
            for data in iterator:
                if data is None:
                    break
                record_writer(xf, serializer, header, data, options, **kwargs)


def toxml(iterator, destination, encoding='utf-8', serializer=None, **kwargs):
    """Потоковая генерация XML"""

    if serializer is None:
        serializer = simple_serializer

    try:
        with etree.xmlfile(destination, encoding=encoding) as xf:
            xf.write_declaration()
            model_writer(xf, serializer, iterator, **kwargs)

    # FIXME: lxml падает при закрытии большого файла
    except etree.SerialisationError:
        pass


def get_packet(rules, files_destination, file_name=None, prefix=None, **kwargs):
    """Формирует XML с данными и возвращает путь до неё
    :param files_destination: путь по кторому хранятся файлы
    :return: /full/path/to/xml,
    """

    if file_name is None:
        file_name = "{MNEMONIC}_{DATETIME}.xml".format(
            MNEMONIC=prefix,
            DATETIME=datetime.now().strftime("%Y%m%d_%H%M%S"))

    xml_filename = os.path.join(files_destination, file_name)

    serializer = None

    if callable(rules):
        rules_config = rules(**kwargs)
    else:
        rules_config = rules

    if isinstance(rules_config, dict):
        packet_mapping = rules_config['packet_mapping']
        serializer = rules_config.get('serializer')
    else:
        packet_mapping = rules

    toxml(process_mapping(packet_mapping), xml_filename,
          serializer=serializer, **kwargs)

    return xml_filename
