# -*- coding: utf-8 -*-

u"""Клиентская часть системы для взаимодействия со СМЭВ3."""

# Vendor Library
import requests

# Project Library
from ..files import FilesContainer
from ..security.sign import EPOVOpenSSLSigner
from .send_request import OpenSSLSendRequestRequest
from .send_request import SendRequestResponse
from .get_response import GetResponseRequest
from .get_response import GetResponseResponse
from .get_incoming_queue_statistics import (
    OpenSSLGetIncomingQueueStatisticsRequest,
    GetIncomingQueueStatisticsResponse)
from .get_request import (
    GetRequestRequest,
    GetRequestResponse)
from .send_response import (
    SendResponseRequest,
    SendResponseResponse,
    OpenSSLSendResponseRequest)
from .get_status import (
    OpenSSLGetStatusRequest)
from .ack import (
    OpenSSLAckRequest,
    AckResponse)


class Client(object):
    u"""Класс, отвечает за отправку данных."""

    def __init__(
            self, url, is_test=False,
            need_verified_response=True,
            storage_path=u"/tmp/"):
        self.url = url
        self.is_test = is_test
        self._need_verified_response = need_verified_response
        self._storage_path = storage_path
        self._private_data = {}

    def log(self, request, request_text, response, response_text, error_text):
        u"""Метод для реализации логирования"""

        pass

    def send_request(self, data, files=None):
        u"""Метод выполняет СМЭВ3 метод SendRequest

        :param data: lxml.Element с бизнес данными.
        :param files: Итерируемый объект с путями до файлов.
        """

        file_container = FilesContainer(files=files)
        request = OpenSSLSendRequestRequest(
            data, files=file_container, is_test=self.is_test)
        response = self._send(
            request, SendRequestResponse,
            need_verified=self._need_verified_response)

        return request, response

    def get_response(
            self, namespace=None, root_element=None, sending_timestamp=None):
        u"""Метод выполняет СМЭВ3 метод GetResponse.

        :param namespace:
        :param root_element:
        :param sending_timestamp: Время отправки SendRequest.
        """

        get_response_request = GetResponseRequest(
            namespace, root_element, sending_timestamp,
            signer=EPOVOpenSSLSigner()
        )
        return get_response_request, self._send(
            get_response_request, GetResponseResponse,
            need_verified=self._need_verified_response)

    def send_response(self, to, data, files=None):
        u"""Метод выполняет СМЭВ3 метод SendResponse

        :param data: lxml.Element с бизнес данными.
        :param files: Итерируемый объект с путями до файлов.
        """

        file_container = FilesContainer(files=files)
        request = OpenSSLSendResponseRequest(
            to, data, files=file_container, is_test=self.is_test)
        response = self._send(
            request, SendResponseRequest,
            need_verified=self._need_verified_response)

        return request, response

    def get_request(
            self, namespace_uri, root_element, sending_timestamp):
        u"""Метод выполняет СМЭВ3 метод GetRequest.

        :param namespace:
        :param root_element:
        :param sending_timestamp: Время отправки SendRequest.
        """

        get_request_request = GetRequestRequest(
            namespace_uri, root_element, sending_timestamp,
            signer=EPOVOpenSSLSigner()
        )
        return get_request_request, self._send(
            get_request_request, GetRequestResponse,
            need_verified=self._need_verified_response)

    def ack(self, message_id, accepted="true"):
        u"""Метод выполняет отправку СМЭВ3 метода Ack.

        :param message_id: Идентификатор запроса, которое будет подтверждено.
        :param accepted: Флаг - подтвержден/не подтвержден в виде строки.
        """

        ack_request = OpenSSLAckRequest(message_id, accepted)
        return ack_request, self._send(
            ack_request, AckResponse, need_verified=False)

    def get_incoming_queue_statistics(self, timestamp):
        u"""Метод выполняет отправку СМЭВ3 метода GetIncomingQueueStatistic.

        :param timestamp:
        """
        get_incoming_queue_request = OpenSSLGetIncomingQueueStatisticsRequest(
            timestamp)
        return get_incoming_queue_request, self._send(
            get_incoming_queue_request, GetIncomingQueueStatisticsResponse, need_verified=False)

    def get_status(self, timestamp):
        u"""Метод выполняет отправку СМЭВ3 метода GetStatus.

        :param timestamp:
        """
        get_status_request = OpenSSLGetStatusRequest(
            timestamp)
        return get_status_request, self._send(
            get_status_request, None, need_verified=False)

    def _send(self, request, response_cls, need_verified=True):
        u"""Отправка данных.

        :param request: Запрос в виде lxml объекта.
        :param response_cls: Класс для представления ответа.
        :param need_verified: Необходимо ли проверять подпись ответа.
        """

        request_text = str(request)

        response = requests.post(
            self.url, request_text, headers={"SOAPAction": request.method})

        # Отложенное исключение
        ex = None

        if response.status_code != 200:
            ex = ValueError(
                "Response status code is {0}.".format(response.status_code))

        response_text = response.text.encode("utf-8")
        response_object = None

        if not ex:
            try:
                response_object = response_cls(response_text)
            except Exception as ex:
                # Сохраняем исключение в ex для дальнейшего использования
                pass

        # Проверка подлинности ответа.
        if not ex and need_verified and not response_object.verified:
            not_verified_msg = "{0} not verified.".format(
                response_cls.__name__)
            ex = ValueError(not_verified_msg)

        if request.loggable:
            self.log(request, request_text, response_object, response_text, ex and ex.message or '')

        # Выкидываем отложенное исключение если были ошибки
        if ex:
            raise ex

        return response_object
