# -*- coding: utf-8 -*-

# Standard Library
from functools import partial

# Project Library
from ..security.sign import EPOVOpenSSLSigner
from ..element_maker import E_soap_env
from ..element_maker import E_ns1
from ..element_maker import E_ns
from ..namespaces import ns1
from ..namespaces import ns
from .base import BaseRequest
from .base import BaseResponse
from .models import RequestTypes


class AckRequest(BaseRequest):
    u"""Запрос метода Ack.

    Метод используется для подтверждения клиентом, того факта, что система
    обработала ответ сервиса.
    """

    # Шаблон запроса
    request_template = E_soap_env.Envelope(
        E_soap_env.Header(),
        E_soap_env.Body(
            E_ns.AckRequest(
                E_ns1.AckTargetMessage()
            )
        )
    )

    method = RequestTypes.Ack

    loggable = True

    def __init__(self, message_id, accepted="true", *args, **kwargs):
        u"""Конструктор.

        :param message_id: Идентификатор сообщения.
        """
        super(AckRequest, self).__init__(*args, **kwargs)

        self._message_id = message_id

        if accepted not in ("true", "false"):
            raise ValueError(
                "AckRequest error. Accepted attribute must be true or false.")
        self._accepted = accepted
        self._ack_target_message = None

    @property
    def ack_target_message(self):
        u"""Объект, который содержит message_id."""

        if self._ack_target_message is None:
            self._ack_target_message = self._request.find(
                ".//{{{0}}}AckTargetMessage".format(ns1))

        return self._ack_target_message

    def _set_data(self):
        u"""Установка параметров запроса.

        Устанавливаем в запрос message_id.
        """
        super(AckRequest, self)._set_data()

        self.ack_target_message.text = self._message_id
        self.ack_target_message.set("accepted", self._accepted)

    def _sign(self):
        u"""Метод выполняет подпись элементов конверта."""
        super(AckRequest, self)._sign()

        caller_information_system_signature = self._signer.sign(
            self.ack_target_message)

        ack_request_tag = self._request.find(
            ".//{{{0}}}AckRequest".format(ns))
        ack_request_tag.append(caller_information_system_signature)

    @property
    def message_id(self):
        return self._message_id


# Запрос подписанный при помощи OpenSSL.
OpenSSLAckRequest = partial(
    AckRequest,
    signer=EPOVOpenSSLSigner()
)


class AckResponse(BaseResponse):
    u"""Ответ метода AckRequest.

    Метод используется для подтверждения обработки ответа,
    результата не возвращает.
    """
