# -*- coding: utf-8 -*-

import copy

from lxml import etree

from ..security.sign import MockSigner


class PrettyViewMixin(object):
    u"""Mixin, добавляющий в класс метод для получения читаемого ответа.

    Запрос или ответ могут содержать блоки с данными большого размера.
    Например файлы. Такая информация является излишней при отображении
    пользователю.
    """

    def _get_object_for_view(self):
        u"""Метод возвращает объект в удобном для просмотра представлении."""
        raise NotImplementedError()

    @property
    def pretty(self):
        u"""Метод представляет запрос/ответ в удобном для чтения виде."""
        view_object = self._get_object_for_view()
        return etree.tostring(view_object, pretty_print=True)


class BaseRequest(PrettyViewMixin):
    u"""Базовый класс для запроса к СМЭВ3 со стороны потребителя.

    Класс содержит логику для создания запроса.
    Класс имеет шаблон запроса и знает какие параметры в какой тэг положить.
    Класс знает какие элементы необходимо подписать.
    Класс управляет процессом создания подписи, но формирования служебного
    XML блока делегирует экземпляру класса подписчика.
    Класс предоставляет свойство с объектным представлением запроса request.
    """

    # Шаблон запроса. Заполняется в классах-наследниках.
    request_template = None

    # Имя метода
    method = None

    # Логируется ли данный запрос/ответ
    loggable = False

    # Словарь функций обратного вызова для обработки ответов
    callbacks = None

    def __init__(self, signer=None):
        u"""Конструктор.

        :param signer: Инстанс класса выполняющего подпись элементов конверта.
        """
        self._signer = signer or MockSigner()

        self._request = None

        if self.method is None:
            raise ValueError("Class attribute method must be set.")

    def _assert_request(self):
        assert self._request is not None, "Request must be initialized"

    def _sign(self):
        u"""Метод выполняет попись элементов конверта.

        Логика подписывания определяется наследниками.
        """
        self._assert_request()

    def _set_data(self):
        u"""Установка параметров запроса.

        Логика заполнения определяется наследниками.
        """
        self._assert_request()

    def _prepare_request(self):
        u"""Дополнительная логика при подготовки запроса.

        Переопределяется в классах-наследниках.
        """
        self._assert_request()

    @property
    def request(self):
        u"""Готовое объектное представление XML запроса."""

        if self._request is None:
            self._request = copy.deepcopy(self.request_template)

            self._set_data()
            self._prepare_request()
            self._sign()

        return self._request

    def __str__(self):
        u"""Строковое представление запроса."""
        return etree.tostring(self.request, encoding='utf-8')

    def _get_object_for_view(self):
        u"""Метод возвращает объект в удобном для просмотра представлении."""
        return self.request


class BaseResponse(PrettyViewMixin):
    u"""Базовый класс для ответа от СМЭВ3.

    Класс предоставляет набор свойств для извлечения необходимой информации
    из ответа.
    Класс управляет процессом проверки подлинности ответа.
    Работа по определению подлинности подписи конкретного элемента делигируется
    специальному классу.
    """

    def __init__(self, response):
        u"""Конструктор.

        :param response: Строковое представление ответа.
        :raises: etree.XMLSyntaxError
        """
        self._response = etree.fromstring(response)

    @property
    def verified(self):
        u"""Проверка подлинности ответа.

        Конкретная реализация в классах наследниках.
        """
        return True

    def _get_object_for_view(self):
        u"""Метод возвращает объект в удобном для просмотра представлении."""
        return self._response
