# -*- coding: utf-8 -*-

import copy

from ..element_maker import E_soap_env
from ..element_maker import E_ns1
from ..element_maker import E_ns
from ..namespaces import ns1
from ..namespaces import ns
from ..files import files_iterator
from ..security.verify import verify_files
from ..helpers import get_tag_content
from .base import BaseRequest
from .base import BaseResponse
from .models import RequestTypes


class GetRequestRequest(BaseRequest):
    u"""Запрос метода GetRequest.

    Метод используется для опроса СМЭВ3 на наличие в очереди необработанных запросов.
    Предназначение класса - предоставить текст запроса для отправки в СМЭВ.
    """

    # Шаблон запроса
    request_template = E_soap_env.Envelope(
        E_soap_env.Header(),
        E_soap_env.Body(
            E_ns.GetRequestRequest(
                E_ns1.MessageTypeSelector(
                    E_ns1.NamespaceURI(),
                    E_ns1.RootElementLocalName(),
                    E_ns1.Timestamp()
                )
            )
        )
    )

    method = RequestTypes.GetRequest

    loggable = True

    def __init__(
            self, namespace_uri, root_element, timestamp, *args, **kwargs):
        u"""Конструктор.

        :param namespace_uri: Параметр, по которому будет происходить
        фильтрация.
        :param root_element: Корневой элемент бизнес-части запроса.
        :param timestamp: Время доставки сообщения с ответом на сервер СМЭВ3.
        :param signer: Инстанс класса выполняющего подпись конверта.
        """

        super(GetRequestRequest, self).__init__(*args, **kwargs)

        self._namespace_uri = namespace_uri
        self._root_element = root_element
        self._timestamp = timestamp

    def _set_data(self):
        u"""Подставляет данные в запрос."""

        super(GetRequestRequest, self)._set_data()

        namespace_uri_tag = self._request.find(
            ".//{{{0}}}NamespaceURI".format(ns1))
        if self._namespace_uri is None:
            namespace_uri_tag.getparent().remove(namespace_uri_tag)
        else:
            namespace_uri_tag.text = self._namespace_uri

        root_element_tag = self._request.find(
            ".//{{{0}}}RootElementLocalName".format(ns1))
        if self._root_element is None:
            root_element_tag.getparent().remove(root_element_tag)
        else:
            root_element_tag.text = self._root_element

        timestamp_tag = self._request.find(".//{{{0}}}Timestamp".format(ns1))
        timestamp_tag.text = self._timestamp

    def _sign(self):
        u"""Метод выполняет подпись элементов запроса."""

        super(GetRequestRequest, self)._sign()

        message_type_selector = self._request.find(
            ".//{{{0}}}MessageTypeSelector".format(ns1))
        caller_information_system_signature = self._signer.sign(
            message_type_selector)

        get_response_request = self._request.find(
            ".//{{{0}}}GetRequestRequest".format(ns))
        get_response_request.append(caller_information_system_signature)


class GetRequestResponse(BaseResponse):
    u"""Ответ в методе GetRequest."""

    def get_files(self, storage_path, file_prefix):
        u"""Возвращает файлы приложенные к конверту."""

        attachment_header_list = self._response.find(
            ".//{{{0}}}AttachmentHeaderList".format(ns1))

        attachment_content_list = self._response.find(
            ".//{{{0}}}AttachmentContentList".format(ns1))

        if not (attachment_header_list and attachment_content_list):
            return []

        if not verify_files(attachment_header_list, attachment_content_list):
            raise ValueError("Verify files failed")

        return files_iterator(attachment_content_list, storage_path, file_prefix)

    @property
    def content(self):
        u"""Возвращает данные запроса.

        :return: lxml Element
        """
        return self._response.find(
            ".//{{{0}}}MessagePrimaryContent/*".format(ns1))

    @property
    def message_id(self):
        u"""Возвращает MessageId готового запроса."""
        return get_tag_content(
            self._response.find(
                ".//{{{0}}}MessageID".format(ns)))

    @property
    def reply_to(self):
        u"""Возвращает ReplyTo готового запроса."""
        return get_tag_content(
            self._response.find(
                ".//{{{0}}}ReplyTo".format(ns)))

    def _get_object_for_view(self):
        u"""Метод возвращает объект в удобном для просмотра представлении."""
        view_response = copy.deepcopy(self._response)
        files_content_elements = view_response.findall(
            ".//{{{0}}}Content".format(ns1))
        for element in files_content_elements:
            parent_element = element.getparent()
            parent_element.remove(element)

        return view_response
