# -*- coding: utf-8 -*-

# Standard Library
import copy
import uuid
import base64
import shutil
import StringIO
from functools import partial

# Project Library
from ..security.sign import file_signer
from ..security.sign import EPCPOpenSSLSigner
from ..security.sign import EPOVOpenSSLSigner
from ..element_maker import E_soap_env
from ..element_maker import E_ns1
from ..element_maker import E_ns
from ..namespaces import ns1
from ..namespaces import ns
from ..helpers import get_tag_content
from .base import BaseRequest
from .base import BaseResponse
from .models import RequestTypes


class SendRequestRequest(BaseRequest):
    u"""Запрос метода SendRequest.

    Метод используется для отправки запроса в ИС-поставщик услуг через СМЭВ3.
    Предназначение класса - предоставить текст запроса для отправки в СМЭВ3.
    """

    # Шаблон запроса
    request_template = E_soap_env.Envelope(
        E_soap_env.Header(),
        E_soap_env.Body(
            E_ns.SendRequestRequest(
                E_ns.SenderProvidedRequestData(
                    E_ns.MessageID(),
                    # Блок структурированных сведений в соответствии с
                    # требованиями поставщика.
                    E_ns1.MessagePrimaryContent()
                )
            )
        )
    )

    method = RequestTypes.SendRequest

    loggable = True

    def __init__(
            self, primary_content, files=None,
            need_personal_signature=False,
            ep_cp_signer=None,
            is_test=False,
            *args, **kwargs):
        u"""Конструктор.

        :param primary_content: etree.Element представления запроса.
        :param files: Файлы прилагаемые к запросу.
        :param need_personal_signature: Необходимость ЭП-СП.
        :param ep_cp_signer: Класс выполняющий ЭП-СП подпись.
        :param is_test: Признак тестового взаимодействия.
        """
        super(SendRequestRequest, self).__init__(*args, **kwargs)

        if primary_content is None:
            raise ValueError("Bad content")

        self._primary_content = primary_content
        self._files = files or ()
        self._need_personal_signature = need_personal_signature
        if self._need_personal_signature and ep_cp_signer is None:
            raise ValueError("EP-CP signer must be specified")
        self._ep_cp_signer = ep_cp_signer

        # Признак тестового взаимодействия.
        self._is_test = is_test

        # Объектное представление запроса.
        self._message_id = None

    def _set_data(self):
        u"""Подставляет данные в запрос."""

        super(SendRequestRequest, self)._set_data()

        message_primary_content = self._request.find(
            ".//{{{0}}}MessagePrimaryContent".format(ns1))
        message_primary_content.append(self._primary_content)

    def _attach_files(self):
        u"""Добавление файлов к запросу.

        Добавление файлов заключается в формировании тегов AttachmentContent и
        AttachmentHeader.
        """

        self._assert_request()

        if not self._files:
            return

        # Создание контейнера для хранения подписей приложенных файлов.
        sender_provided_request_data = self._request.find(
            ".//{{{0}}}SenderProvidedRequestData".format(ns))
        attachment_header_list = E_ns1.AttachmentHeaderList()
        sender_provided_request_data.append(attachment_header_list)

        # Создание контейнера для хранения содержимого приложенных файлов.
        sender_request_request = self._request.find(
            ".//{{{0}}}SendRequestRequest".format(ns))
        attachment_content_list = E_ns1.AttachmentContentList()
        sender_request_request.append(attachment_content_list)

        for _file in self._files:

            file_id, file_mime, file_path = (
                _file.get("id"),
                _file.get("mime"),
                _file.get("path")
            )

            file_content = StringIO.StringIO()
            with open(file_path, "r") as fd:
                shutil.copyfileobj(fd, file_content)

            # Формируем AttachmentContent
            _id = E_ns1.Id()
            _id.text = file_id

            content = E_ns1.Content()
            content.text = base64.b64encode(file_content.getvalue())

            attachment_content = E_ns1.AttachmentContent()
            attachment_content.append(_id)
            attachment_content.append(content)
            attachment_content_list.append(attachment_content)

            # Формирование AttachmentHeader
            attachment_header = file_signer(file_id, file_mime, file_content)
            attachment_header_list.append(attachment_header)

    def _set_uuid(self):
        u"""Установка идентификатора сообщения."""
        message_id = self._request.find(".//{{{0}}}MessageID".format(ns))
        message_id.text = self.message_id

    def _set_test_flag(self):
        u"""Выставление тестового признака.

        Метод добавляет в запрос тестовый признак,
        если это задано при инициализации (is_test)
        """
        if self._is_test:
            send_request_request = self._request.find(
                ".//{{{0}}}SenderProvidedRequestData".format(ns))
            send_request_request.append(E_ns.TestMessage())

    def _sign(self):
        u"""Метод выполняет подпись запроса."""

        super(SendRequestRequest, self)._sign()

        sender_provided_request_data = self._request.find(
            ".//{{{0}}}SenderProvidedRequestData".format(ns))

        # Если поставщик требует отдельной подписи для MessagePrimaryContent
        if self._need_personal_signature:
            message_primary_content = self._request.find(
                ".//{{{0}}}MessagePrimaryContent".format(ns1))
            personal_signature = self._ep_cp_signer.sign(
                message_primary_content.getchildren()[0],
                root_name="PersonalSignature")

            attachment_header_list = self._request.find(
                ".//{{{0}}}AttachmentHeaderList".format(ns1))
            # Если в документе уже есть AttachmentHeaderList, то
            # PersonalSignature должен быть перед ним.
            if attachment_header_list:
                attachment_header_list_idx = (
                    sender_provided_request_data.getchildren().index(
                        attachment_header_list)
                )
                sender_provided_request_data.insert(
                    attachment_header_list_idx,
                    personal_signature)
            # В противном случае он последний.
            else:
                sender_provided_request_data.append(personal_signature)

        caller_information_system_signature = self._signer.sign(
            sender_provided_request_data)
        send_request_request = self._request.find(
            ".//{{{0}}}SendRequestRequest".format(ns))
        send_request_request.append(caller_information_system_signature)

    def _prepare_request(self):
        u"""Дополнительная логика при формировании запроса."""
        super(SendRequestRequest, self)._prepare_request()

        self._attach_files()
        self._set_test_flag()
        self._set_uuid()

    @property
    def files_paths(self):
        u"""Свойство содержит строку с путями до файлов."""
        if self._files:
            return self._files.paths
        else:
            return []

    @property
    def message_id(self):

        if self._message_id is None:
            self._message_id = str(uuid.uuid1())
        return self._message_id

    def _get_object_for_view(self):
        u"""Метод возвращает объект в удобном для просмотра представлении."""
        view_request = copy.deepcopy(self.request)
        files_content_elements = view_request.findall(
            ".//{{{0}}}Content".format(ns1))
        for element in files_content_elements:
            parent_element = element.getparent()
            parent_element.remove(element)

        return view_request


# Запрос подписанный при помощи OpenSSL.
OpenSSLSendRequestRequest = partial(
    SendRequestRequest,
    signer=EPOVOpenSSLSigner(),
    ep_cp_signer=EPCPOpenSSLSigner())


class SendRequestResponse(BaseResponse):
    u"""Ответ метода SendRequest.

    Метод используется для отправки запроса в ИС-поставщика услуг через СМЭВ.
    Предназначение класса - предоставить интерфейс для извлечение информации
    из объектного представления ответа.
    """

    @property
    def sending_timestamp(self):
        u"""Свойство содержит информацию о времени доставке сообщения."""

        return get_tag_content(
            self._response.find(
                ".//{{{0}}}SendingTimestamp".format(ns)))
