# -*- coding: utf-8 -*-

import os
import base64
import mimetypes

from .namespaces import ns1
from .helpers import get_tag_content


class FilesContainer(object):
    u"""Класс контейнер для хранения информации о файлах.

    Класс используется для получения информации о файлах при формировании
    SOAP конвертов.
    """

    def __init__(self, files):
        u"""Конструктор.

        :param files: Список путей до файлов.
        """
        self._files = []

        if files:
            self.extend(files)

    def extend(self, files):
        u"""Добавление в контейнер файлов.

        :param files: Список имен файлов.
        """

        for _file in files:
            self.append(_file)

    def append(self, file_path):
        u"""Добавление в контейнер файла.

        :param file_path: Путь до файла, что будет добавлен в контейнер.
        """
        if not os.path.exists(file_path):
            raise ValueError("File {0} does not exist".format(file_path))

        file_name = os.path.basename(file_path)
        _, file_extension = os.path.splitext(file_name)

        try:
            mime = mimetypes.types_map[file_extension]
        except KeyError:
            raise ValueError("File {0} unknown mime type".format(file_path))

        self._files.append({
            "id": file_name,
            "mime": mime,
            "path": file_path
        })

    @property
    def paths(self):
        u"""Возвращает генератор путей до файлов."""
        return (file_["path"] for file_ in self._files)

    def __iter__(self):
        return iter(self._files)

    def __len__(self):
        return len(self._files)


def files_iterator(attachment_content_list, storage_path, file_prefix):
    u"""Итератор по файлам в конверте.

    :param attachment_content_list: XML блок содержащий приложенные файлы.
    """

    for attachment_content in attachment_content_list.getchildren():
        file_id = get_tag_content(
            attachment_content.find(".{{{0}}}Id".format(ns1)))
        content = get_tag_content(
            attachment_content.find(".{{{0}}}Content".format(ns1)))

        if not (file_id and content):
            # TODO: Возможно стоит райзить ошибку.
            continue

        # Сохраняем на жесткий диск и отдадим ссылку пользователю.
        stored_file_name = os.path.join(storage_path, file_prefix + file_id)
        with open(stored_file_name, "w") as f:
            f.write(base64.decodestring(content))

        yield stored_file_name
