# -*- coding: utf-8 -*-

# Standard Library
import re
from collections import defaultdict

# Vendor Library
from lxml import etree
from lxml.builder import ElementMaker


def normalize(_xml):
    u"""Нормализация XML по СМЭВ.

    Метод нормализации urn://smev-gov-ru/xmldsig/transform.
    :param _xml: Строковое представление XML.
    """

    # Результирующая XML-ка
    result = None

    # Стэк
    stack = []
    level = 0

    # Словарь содержит соответствие текущего доступного ns и его числового
    # эквивалента в нормализованном представлении (http://test/0 -> 2).
    # В нормализованном представлении этому uri будет соответствовать
    # префикс ns2.
    current_elements_uri = {}
    # Словарь содержит соответствие текущего доступного ns и элемента,
    # который его туда добавил.
    elements_uri_producer = {}
    # Аналог current_elements_uri, но для префиксов атрибутов.
    current_elements_attributes_uri = {}
    # Соответствие tag -> [ns атрибутов, которые он добавил на своем уровне]
    elements_attributes_uri_producer = defaultdict(list)
    prefix_count = 1

    # regex шаблоны для поиска.
    qualified_attribute_pattern = re.compile(r"^{http:\/\/.*}.*$")
    qualified_attribute_ns = re.compile(r"(?<={)http:\/\/.*(?=})")
    qualified_attribute_tag = re.compile(r"(?<=}).*")

    for event, src_element in etree.iterparse(
            _xml, events=("start", "end")):

        if event == "start":
            ns_uri = src_element.nsmap[src_element.prefix]

            # Разделение атрибутов на две группы
            # qualified и unqualified
            qualified_form_attributes = []
            unqualified_form_attributes = []
            for attr_name, attr_value in src_element.items():
                if qualified_attribute_pattern.match(attr_name):
                    separated_attr_name = (
                        qualified_attribute_ns.search(attr_name).group(0),
                        qualified_attribute_tag.search(attr_name).group(0)
                    )
                    qualified_form_attributes.append(
                        (separated_attr_name, attr_value))
                else:
                    unqualified_form_attributes.append((attr_name, attr_value))

            # 7 пункт нормализации.
            # Атрибуты должны быть отсортированы.
            # qualified атрибуты сортируются по uri, а потом по имени.
            sorted_qualified_attributes = sorted(qualified_form_attributes)
            # unqualified атрибуты сортируются по имени.
            sorted_unqualified_attributes = sorted(unqualified_form_attributes)

            # Флаги необходимости регистрировать новые префиксы.
            need_declared_ns = False
            need_declared_attributes_ns = False
            # 6 пункт нормализации.
            # Замена префиксов на автоматически сгенерированные.
            # <pref:tag>123</pref:tag> -> <ns1:tag>123</ns1:tag>
            if ns_uri not in current_elements_uri:
                need_declared_ns = True
                current_elements_uri[ns_uri] = prefix_count
                elements_uri_producer[ns_uri] = src_element.tag
                prefix_count += 1

            for (_attr_uri,
                 attr_tag), _attr_value in sorted_qualified_attributes:
                # Сначало проверим, что такой ns не используется
                # для элемента или другого атрибута.
                if (_attr_uri not in current_elements_attributes_uri and
                        _attr_uri not in current_elements_uri):
                    need_declared_attributes_ns = True
                    current_elements_attributes_uri[_attr_uri] = prefix_count
                    elements_attributes_uri_producer[src_element.tag].append(
                        _attr_uri)
                    prefix_count += 1

            # Формирование параметров для создания XML элемента.
            element_params = dict(namespace=ns_uri)
            # Сначало добавляются ns элемента
            if need_declared_ns:
                element_params["nsmap"] = {
                    "ns{0}".format(current_elements_uri[ns_uri]): ns_uri
                }
            # После них ns атрибутов.
            if need_declared_attributes_ns:
                if "nsmap" not in element_params:
                    element_params["nsmap"] = {}
                for attr_uri in sorted(current_elements_attributes_uri):
                    element_params["nsmap"]["ns{0}".format(
                        current_elements_attributes_uri[attr_uri])
                    ] = attr_uri

            # Инстанцирование нового элемента.
            dst_element = getattr(
                ElementMaker(**element_params),
                src_element.tag
            )()

            # 2 пункт нормализации.
            # Убираем все пробельные символы и символы табуляции.
            if src_element.text:
                if src_element.text.isspace():
                    dst_element.text = src_element.text.strip()
                else:
                    dst_element.text = src_element.text
                    dst_element.text = dst_element.text.replace("\n", "")

            # 7 пункт нормализации.
            # Добавляем отсортированные атрибуты в выходной тег.
            # Сначало идут сортированные элементы в qualified form.
            for (_attr_uri,
                 attr_tag), _attr_value in sorted_qualified_attributes:
                dst_element.set("{{{0}}}{1}".format(
                    _attr_uri, attr_tag), _attr_value)
            # После них в unqualified form.
            for attr, value in sorted_unqualified_attributes:
                dst_element.set(attr, value)

            # Добавление элемента в выходную XML.
            if result is None:
                result = dst_element
            else:
                if level:
                    stack[-1].append(dst_element)

            stack.append(dst_element)

            level += 1
        elif event == "end":
            # Элемент в котором задекларирован ns закрывается и мы
            # удаляем ns из словаря.
            ns_uri = src_element.nsmap[src_element.prefix]

            if (ns_uri in current_elements_uri and
                    src_element.tag == elements_uri_producer[ns_uri]):
                del current_elements_uri[ns_uri]
                del elements_uri_producer[ns_uri]

            attributes_uri = elements_attributes_uri_producer[src_element.tag]
            for attribute_uri in attributes_uri:
                del current_elements_attributes_uri[attribute_uri]
            del elements_attributes_uri_producer[src_element.tag]

            level -= 1
            stack.pop()

    # 3 пункт нормализации.
    # Пустые теги <tag/> заменяются на <tag></tag>
    return etree.tostring(result, method="html", encoding='utf-8')
