# -*- coding: utf-8 -*-

import uuid
import base64
from functools import partial
from StringIO import StringIO

from django.conf import (
    settings)

from lxml import etree
# from django.conf import settings
from M2Crypto import BIO, SMIME
from spyne_smev.crypto import sign
from spyne_smev.crypto import get_text_digest
from spyne_smev.crypto import get_signature_algorithm_name
from spyne_smev.wsse.utils import _get_clean_cert_data
from spyne_smev.wsse.utils import _signature_method_exclusions

from ..normalize import normalize
from ..element_maker import E_ns
from ..element_maker import E_ns1
from ..element_maker import E_ds, ds


class BaseSigner(object):
    u"""Интерфейс для класса выполняющего подпись.

    Задача класса и его наследников сформировать объектное представление
    служебной части запроса содержащее информацию о ЭЦП.

    Подпись выполняется по алгоритму md_gost94.

    Класс принимает от пользователя lxml объект который необходимо подписать.
    Возвращает lxml объект с подписью пользовательских данных.
    """

    def _get_security_path(self, root_name):
        u"""Функция возвращает объектное представление части XML.

        Функция формирует служебную часть XML, содержащую информацию о ЭЦП.

        :param root_name: Наименование корневого узла.
        """

        return getattr(E_ns, root_name)(
            E_ds.Signature(
                E_ds.SignedInfo(
                    E_ds.CanonicalizationMethod(
                        Algorithm="http://www.w3.org/2001/10/xml-exc-c14n#"),
                    E_ds.SignatureMethod(
                        Algorithm="http://www.w3.org/2001/04/xmldsig-more#"
                                  "gostr34102001-gostr3411"),
                    E_ds.Reference(
                        E_ds.Transforms(
                            E_ds.Transform(
                                Algorithm="http://www.w3.org/2001/10/"
                                          "xml-exc-c14n#"),
                            E_ds.Transform(
                                Algorithm="urn://smev-gov-ru/xmldsig/transform"
                            )
                        ),
                        E_ds.DigestMethod(
                            Algorithm="http://www.w3.org/2001/04/xmldsig-more#"
                                      "gostr3411"),
                        E_ds.DigestValue()
                    )
                ),
                E_ds.SignatureValue(),
                E_ds.KeyInfo(
                    E_ds.X509Data(
                        E_ds.X509Certificate()
                    )
                )
            )
        )

    def sign(self, data, root_name="CallerInformationSystemSignature"):
        u"""Выполняет подпись данных.

        :param data: Данные на основе которых формируется подпись.
        :param root_name: Имя тэга содержащего служебную информацию об ЭЦП
        :return: Возвращает служебную часть запроса, связанную с подписью.
        """
        raise NotImplementedError


class MockSigner(BaseSigner):
    u"""Для тестовых нужд."""

    def sign(self, data, root_name="CallerInformationSystemSignature"):
        pass


class OpenSSLSigner(BaseSigner):
    u"""Подпись данных через OpenSSL."""

    def __init__(self, certificate, private_key):
        u"""Конструктор

        :param certificate: Сертификат.
        """
        self._method = "md_gost94"
        with open(certificate, "r") as f:
            self._certificate = f.read()
        self._private_key = private_key

    def sign(self, data, root_name="CallerInformationSystemSignature"):
        u"""Переопределение для подписи генерируемой OpenSSL.

        :param data: Данные на основе которых формируется подпись.
        :param root_name: Имя тэга содержащего служебную информацию об ЭЦП
        """

        data_id = "Id-{0}".format(str(uuid.uuid4()))
        data.set("Id", data_id)

        security_path = self._get_security_path(root_name)

        # Добавляем сертификат.
        x509certificate = security_path.find(
            ".//{{{0}}}X509Certificate".format(ds))
        x509certificate.text = _get_clean_cert_data(self._certificate)

        c14n = partial(
            etree.tostring, method="c14n",
            exclusive=True, with_comments=False)

        # Укажем ссылку на элемент, чью контрольную сумму вычисляем.
        reference_tag = security_path.find(".//{{{0}}}Reference".format(ds))
        reference_tag.set("URI", "#" + data_id)

        # Заполняем DigestValue.
        digest_value = base64.b64encode(
            get_text_digest(normalize(StringIO(c14n(data))), self._method))
        digest_value_tag = security_path.find(
            ".//{{{0}}}DigestValue".format(ds))
        digest_value_tag.text = digest_value

        # Заполняем SignatureValue.
        signed_info_tag = security_path.find(
            ".//{{{0}}}SignedInfo".format(ds))
        signature_algorithm = get_signature_algorithm_name(self._certificate)
        signature_algorithm = _signature_method_exclusions.get(
            signature_algorithm, signature_algorithm)

        signature_value = base64.b64encode(
            sign(
                c14n(signed_info_tag), self._certificate, self._private_key,
                signature_algorithm
            )
        )
        signature_value_tag = security_path.find(
            ".//{{{0}}}SignatureValue".format(ds))
        signature_value_tag.text = signature_value

        return security_path


# Электронная подпись органов власти (ЭП-ОВ).
# С помощью ЭП-ОВ обеспечивается целостность запроса и идентификация
# ИС отправителя.
EPOVOpenSSLSigner = partial(
    OpenSSLSigner,
    settings.SMEV3_CERTIFICATE,
    settings.SMEV3_PASSWORD
)

# Электронная подпись должностного лица (ЭП-СП).
EPCPOpenSSLSigner = partial(
    OpenSSLSigner,
    settings.SMEV3_CERTIFICATE,
    settings.SMEV3_PASSWORD
)


class CryptoProSigner(BaseSigner):
    u"""Подпись конверта через Крипто Про."""


def file_signer(file_id, mime_type, file_content):
    u"""Функция возвращает служебную часть XML c ЭЦП приложенного файла.

    Генерация сертификата
    openssl req -newkey rsa:1024 -nodes -x509 -days 365 -out signer.pem
    mv privkey.pem signer_key.pem
    :param file_id: Идентификатор файла. Может совпадать с именем.
    :param mime_type: MimeType файла.
    :param file_content: Путь до файла.
    """

    mime = SMIME.SMIME()
    # M2Crypto не умеет работать с unicode строками.
    mime.load_key(
        str(settings.SMEV3_PRIVATE_KEY),
        str(settings.SMEV3_CERTIFICATE))

    pkcs7 = mime.sign(
        BIO.MemoryBuffer(file_content.getvalue()),
        SMIME.PKCS7_DETACHED | SMIME.PKCS7_BINARY)
    pkcs7_buf = BIO.MemoryBuffer()
    pkcs7.write_der(pkcs7_buf)

    return E_ns1.AttachmentHeader(
        E_ns1.contentId(file_id),
        E_ns1.MimeType(mime_type),
        E_ns1.SignaturePKCS7(base64.b64encode(pkcs7_buf.read()))
    )
