# -*- coding: utf-8 -*-

import base64
from StringIO import StringIO
from functools import partial

from lxml import etree
from spyne_smev.crypto import verify
from spyne_smev.crypto import get_text_digest

from ..namespaces import ds
from ..normalize import normalize


def verify_files(attachment_header_list, attachment_content_list):
    u"""Функция проверяет корректность подписи файлов.

    :param attachment_header_list: Служебный XML блок с данными о подписях.
    :param attachment_content_list: Блок с контентом файлов.
    """

    return True


def is_verified_data(data_element, security_element):
    u"""Функция проверяет подпись элемента.

    :param data_element: Элемент который подписали.
    :param security_element: Служебный элемент с подписью.
    """

    c14n = partial(
        etree.tostring, method="c14n",
        exclusive=True, with_comments=False)

    calculated_digest_value = base64.b64encode(
        get_text_digest(normalize(StringIO(c14n(data_element))), "md_gost94"))
    envelope_digest_value = security_element.find(
        ".//{{{0}}}DigestValue".format(ds))

    if envelope_digest_value is None:
        raise ValueError("Invalid security element structure")
    if calculated_digest_value != envelope_digest_value.text:
        # Хэш не совпадает.
        return False

    signature_value_tag = security_element.find(
        ".//{{{0}}}SignatureValue".format(ds))
    if signature_value_tag is None:
        raise ValueError("SignatureValue tag is required.")

    signature_value = signature_value_tag.text

    signed_info_tag = security_element.find(
        ".//{{{0}}}SignedInfo".format(ds))
    if signed_info_tag is None:
        raise ValueError("SignedInfo tag is required.")

    certificate = security_element.find(
        ".//{{{0}}}X509Certificate".format(ds))
    if certificate is None:
        raise ValueError("X509Certificate tag is required")

    cert = []
    start = 0
    end = step = 64
    while True:
        chunk = certificate.text[start:end]
        if not chunk:
            break
        start += step
        end += step
        cert.append(chunk)

    public_certificate = (
        "-----BEGIN CERTIFICATE-----\n"
        "{0}\n"
        "-----END CERTIFICATE-----".format("\n".join(cert))
    )

    try:
        verify(
            c14n(signed_info_tag),
            public_certificate,
            base64.b64decode(signature_value),
            "md_gost94"
        )
    except ValueError:
        # Нарушена ЭЦП.
        return False

    return True
